@@ -1,3 +1,28 @@
+  /********************************************************************\
+  * BitlBee -- An IRC to other IM-networks gateway                     *
+  *                                                                    *
+  * Copyright 2002-2003 Wilmer van der Gaast and others                *
+  \********************************************************************/
+
+/* The big hairy IRCd part of the project                               */
+
+/*
+  This program is free software; you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation; either version 2 of the License, or
+  (at your option) any later version.
+
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
+
+  You should have received a copy of the GNU General Public License with
+  the Debian GNU/Linux distribution in /usr/share/common-licenses/GPL;
+  if not, write to the Free Software Foundation, Inc., 59 Temple Place,
+  Suite 330, Boston, MA  02111-1307  USA
+*/
+
 #include "bitlbee.h"
 #include "crypting.h"
 
@@ -58,20 +83,49 @@ irc_t *irc_new( int fd )
 	set_add( irc, "to_char", ": ", set_eval_to_char );
 	set_add( irc, "ops", "both", set_eval_ops );
 	
+	irc_write( irc, ":%s NOTICE AUTH :%s", irc->myhost, "BitlBee-IRCd initialized, please go on" );
+	
 	return( irc );
 }
 
 int irc_process( irc_t *irc )
 {
-	char line[IRC_MAX_LINE];
-	int bytes;
+	char *line;
+	int bytes = 0, size = 256, st;
+	struct timeval tv[1];
+	fd_set fds[1];
 	
-	if( ( bytes = read( irc->fd, line, IRC_MAX_LINE ) ) <= 0 )
+	line = malloc( size );
+	
+	FD_ZERO( fds );
+	FD_SET( irc->fd, fds );
+	tv->tv_sec = 0;
+	tv->tv_usec = 0;
+	
+	while( select( irc->fd + 1, fds, NULL, NULL, tv ) > 0 )
 	{
-		return( 0 );
+		st = read( irc->fd, line + bytes, size - bytes );
+		if( st <= 0 )
+		{
+			free( line );
+			return( 0 );
+		}
+		
+		bytes += st;
+		if( bytes == size )
+		{
+			size *= 2;
+			line = realloc( line, size );
+		}
 	}
 	
-	return( irc_process_string( irc, line, bytes ) );
+	if( bytes > 0 )
+		st = irc_process_string( irc, line, bytes );
+	else
+		st = 1;
+	
+	free( line );
+	return( st );
 }
 
 int irc_process_string( irc_t *irc, char *line, int bytes )
@@ -105,7 +159,7 @@ int irc_process_string( irc_t *irc, char *line, int bytes )
 			cmd[arg] = NULL;
 			line[i] = 0;
 			if( !irc_exec( irc, cmd ) ) return( 0 );
-			if( line[i+1] == '\n' ) i ++;
+			if( ( ( i + 1 ) < bytes ) && ( line[i+1] == '\n' ) ) i ++;
 			memset( cmd, 0, sizeof( cmd ) );
 			cmd[0] = line + i + 1;
 			arg = dosplit = 1;
@@ -127,7 +181,7 @@ int irc_exec( irc_t *irc, char **cmd )
 		}
 		else if( irc->user )
 		{
-			irc_reply( irc, 432, "You can't change your nick/user" );
+			irc_reply( irc, 432, "You can't change your nick/userinfo" );
 		}
 		else
 		{
@@ -145,7 +199,12 @@ int irc_exec( irc_t *irc, char **cmd )
 		}
 		else if( irc->nick )
 		{
-			irc_reply( irc, 432, "You can't change your nick/user" );
+			irc_reply( irc, 432, "You can't change your nick/userinfo" );
+		}
+		/* This is not clean, but for now it'll have to be like this... */
+		else if( ( nick_cmp( cmd[1], irc->mynick ) == 0 ) || ( nick_cmp( cmd[1], "nickserv" ) == 0 ) )
+		{
+			irc_reply( irc, 433, "This nick is already in use" );
 		}
 		else if( !nick_ok( cmd[1] ) )
 		{
@@ -192,7 +251,7 @@ int irc_exec( irc_t *irc, char **cmd )
 		}
 		else
 		{
-			if( strcasecmp( cmd[1], irc->nick ) == 0 )
+			if( nick_cmp( cmd[1], irc->nick ) == 0 )
 			{
 				if( cmd[2] )
 					irc_umode_set( irc, irc->nick, cmd[2] );
@@ -209,28 +268,77 @@ int irc_exec( irc_t *irc, char **cmd )
 	}
 	else if( strcasecmp( cmd[0], "PART" ) == 0 )
 	{
+		struct conversation *c;
+		
 		if( !cmd[1] )
 		{
 			irc_reply( irc, 461, "%s :Need more parameters", cmd[0] );
 		}
 		else if( strcasecmp( cmd[1], irc->channel ) == 0 )
 		{
+			user_t *u = user_find( irc, irc->nick );
+			
 			/* Not allowed to leave control channel */
-			irc_write( irc, ":%s!%s@%s PART %s :%s", irc->nick, irc->user, irc->host, irc->channel, cmd[2]?cmd[2]:"" );
-			irc_spawn( irc, user_find( irc, irc->nick ) );
+			irc_part( irc, u, irc->channel );
+			irc_join( irc, u, irc->channel );
+		}
+		else if( ( c = conv_findchannel( cmd[1] ) ) )
+		{
+			user_t *u = user_find( irc, irc->nick );
+			
+			irc_part( irc, u, c->channel );
+			
+			if( c->gc && c->gc->prpl )
+			{
+				c->joined = 0;
+				c->gc->prpl->chat_leave( c->gc, c->id );
+			}
 		}
 		else
 		{
 			irc_reply( irc, 403, "%s :No such channel", cmd[1] );
 		}
 	}
-/*	else if( strcasecmp( cmd[0], "JOIN" ) == 0 )
+	else if( strcasecmp( cmd[0], "JOIN" ) == 0 )
 	{
-		if( cmd[1] )
-			irc_reply( irc, 403, "%s :No such channel", cmd[1] );
+		if( strcasecmp( cmd[1], irc->channel ) == 0 )
+			; /* Dude, you're already there...
+			     RFC doesn't have any reply for that though? */
+		else if( cmd[1] )
+		{
+			if( cmd[1][0] == '#' && cmd[1][1] )
+			{
+				user_t *u = user_find( irc, cmd[1] + 1 );
+				
+				if( u && u->gc && u->gc->prpl && u->gc->prpl->convo_open )
+				{
+					irc_reply( irc, 403, "%s :Initializing groupchat in a different channel", cmd[1] );
+					
+					if( !u->gc->prpl->convo_open( u->gc, u->handle ) )
+					{
+						irc_usermsg( irc, "Could not open a groupchat with %s, maybe you don't have a connection to him/her yet?", u->nick );
+					}
+				}
+				else
+				{
+					irc_reply( irc, 403, "%s :Groupchats are not possible with %s", cmd[1], cmd[1]+1 );
+				}
+			}
+			else
+			{
+				irc_reply( irc, 403, "%s :No such channel", cmd[1] );
+			}
+		}
 		else
 			irc_reply( irc, 461, "%s :Need more parameters", cmd[0] );
-	} */
+	}
+	else if( strcasecmp( cmd[0], "INVITE" ) == 0 )
+	{
+		if( cmd[1] && cmd[2] )
+			irc_invite( irc, cmd[1], cmd[2] );
+		else
+			irc_reply( irc, 461, "%s :Need more parameters", cmd[0] );
+	}
 	else if( strcasecmp( cmd[0], "PRIVMSG" ) == 0 )
 	{
 		if( !cmd[2] )
@@ -265,8 +373,8 @@ int irc_exec( irc_t *irc, char **cmd )
 	}
 	else if( strcasecmp( cmd[0], "QUIT" ) == 0 )
 	{
-//		irc_write( irc, ":%s!%s@%s QUIT :%s", irc->nick, irc->user, irc->host, cmd[1]?cmd[1]:"" );
 		irc_write( irc, "ERROR :%s%s", cmd[1]?"Quit: ":"", cmd[1]?cmd[1]:"Client Quit" );
+		usleep( 100000 );	/* Give the message a bit time */
 		close( irc->fd );
 		return( 0 );
 	}
@@ -330,6 +438,10 @@ int irc_exec( irc_t *irc, char **cmd )
 		/* [TV] This aliases the NS command to PRIVMSG root as well */
 		root_command( irc, cmd + 1 );
 	}
+	else if( strcasecmp( cmd[0], "MOTD" ) == 0 )
+	{
+		irc_motd( irc );
+	}
 	else if( set_getint( irc, "debug" ) )
 	{
 		irc_usermsg( irc, "\002--- Unknown command:" );
@@ -396,36 +508,65 @@ int irc_write( irc_t *irc, char *format, ... )
 
 void irc_names( irc_t *irc, char *channel )
 {
-	user_t *u;
+	user_t *u = irc->users;
 	char *s;
+	int control = ( strcasecmp( channel, irc->channel ) == 0 );
+	struct conversation *c = NULL;
 	
-	u = irc->users;
-	while( u )
+	if( !control )
+		c = conv_findchannel( channel );
+	
+	/* RFC's say there is no error reply allowed on NAMES, so when the
+	   channel is invalid, just give an empty reply. */
+	
+	if( control || c ) while( u )
 	{
 		if( u->online )
 		{
-			if( u->gc )
+			if( u->gc && control )
 			{
-				irc_reply( irc, 353, "@ %s :%s%s", channel, "", u->nick );
+				if( set_getint( irc, "away_devoice" ) && !u->away )
+					s = "+";
+				else
+					s = "";
+				
+				irc_reply( irc, 353, "@ %s :%s%s", channel, s, u->nick );
 			}
-			else
+			else if( !u->gc )
 			{
-				s = "";
 				if( strcmp( u->nick, irc->mynick ) == 0 && ( strcmp( set_getstr( irc, "ops" ), "root" ) == 0 || strcmp( set_getstr( irc, "ops" ), "both" ) == 0 ) )
 					s = "@";
 				else if( strcmp( u->nick, irc->nick ) == 0 && ( strcmp( set_getstr( irc, "ops" ), "user" ) == 0 || strcmp( set_getstr( irc, "ops" ), "both" ) == 0 ) )
 					s = "@";
+				else
+					s = "";
+				
 				irc_reply( irc, 353, "@ %s :%s%s", channel, s, u->nick );
 			}
 		}
+		
 		u = u->next;
 	}
+	
+	/* For non-controlchannel channels (group conversations) only root and
+	   you are listed now. Time to show the channel people: */
+	if( !control && c )
+	{
+		GList *l;
+		
+		for( l = c->in_room; l; l = l->next )
+			if( ( u = user_findhandle( c->gc, l->data ) ) )
+				irc_reply( irc, 353, "@ %s :%s%s", channel, "", u->nick );
+	}
+	
 	irc_reply( irc, 366, "%s :End of /NAMES list", channel );
 }
 
 void irc_who( irc_t *irc, char *channel )
 {
 	user_t *u = irc->users;
+	struct conversation *c;
+	GList *l;
 	
 	if( !channel || *channel == '0' || *channel == '*' || !*channel )
 		while( u )
@@ -440,6 +581,12 @@ void irc_who( irc_t *irc, char *channel )
 				irc_reply( irc, 352, "%s %s %s %s %s %c :0 %s", channel, u->user, u->host, irc->myhost, u->nick, u->away ? 'G' : 'H', u->realname );
 			u = u->next;
 		}
+	else if( ( c = conv_findchannel( channel ) ) )
+		for( l = c->in_room; l; l = l->next )
+		{
+			if( ( u = user_findhandle( c->gc, l->data ) ) )
+				irc_reply( irc, 352, "%s %s %s %s %s %c :0 %s", channel, u->user, u->host, irc->myhost, u->nick, u->away ? 'G' : 'H', u->realname );
+		}
 	else if( ( u = user_find( irc, channel ) ) )
 		irc_reply( irc, 352, "%s %s %s %s %s %c :0 %s", channel, u->user, u->host, irc->myhost, u->nick, u->online ? ( u->away ? 'G' : 'H' ) : 'G', u->realname );
 	
@@ -454,7 +601,7 @@ void irc_login( irc_t *irc )
 	irc_reply( irc,   2, ":Host %s is running BitlBee " BITLBEE_VERSION " " ARCH "/" CPU ".", irc->myhost );
 	irc_reply( irc,   3, ":%s", IRCD_INFO );
 	irc_reply( irc,   4, "%s %s %s %s", irc->myhost, BITLBEE_VERSION, UMODES, CMODES );
-	irc_reply( irc, 422, ":We don't need MOTDs." );
+	irc_motd( irc );
 	irc_umode_set( irc, irc->myhost, "+" UMODE );
 
 	u = user_add( irc, irc->mynick );
@@ -462,7 +609,7 @@ void irc_login( irc_t *irc )
 	u->realname = ROOT_FN;
 	u->online = 1;
 	u->send_handler = root_command_string;
-	irc_spawn( irc, user_find( irc, irc->mynick ) );
+	irc_spawn( irc, u );
 	
 	u = user_add( irc, "NickServ" );
 	u->host = irc->myhost;
@@ -476,7 +623,7 @@ void irc_login( irc_t *irc )
 	u->realname = irc->realname;
 	u->online = 1;
 //	u->send_handler = msg_echo;
-	irc_spawn( irc, user_find( irc, irc->nick ) );
+	irc_spawn( irc, u );
 	
 	bitlbee_init( irc );
 	
@@ -485,12 +632,73 @@ void irc_login( irc_t *irc )
 	irc->status = USTATUS_LOGGED_IN;
 }
 
+void irc_motd( irc_t *irc )
+{
+	int fd;
+	
+	fd = open( MOTDFILE, O_RDONLY );
+	if( fd == -1 )
+	{
+		irc_reply( irc, 422, ":We don't need MOTDs." );
+	}
+	else
+	{
+		char linebuf[80];	/* Max. line length for MOTD's is 79 chars. It's what most IRC networks seem to do. */
+		char *add, max;
+		int len;
+		
+		linebuf[79] = len = 0;
+		max = sizeof( linebuf ) - 1;
+		
+		irc_reply( irc, 375, ":- %s Message Of The Day - ", irc->myhost );
+		while( read( fd, linebuf + len, 1 ) == 1 )
+		{
+			if( linebuf[len] == '\n' || len == max )
+			{
+				linebuf[len] = 0;
+				irc_reply( irc, 372, ":- %s", linebuf );
+				len = 0;
+			}
+			else if( linebuf[len] == '%' )
+			{
+				read( fd, linebuf + len, 1 );
+				if( linebuf[len] == 'h' )
+					add = irc->myhost;
+				else if( linebuf[len] == 'v' )
+					add = BITLBEE_VERSION;
+				else if( linebuf[len] == 'n' )
+					add = irc->nick;
+				else
+					add = "%";
+				
+				strncpy( linebuf + len, add, max - len );
+				while( linebuf[++len] );
+			}
+			else if( len < max )
+			{
+				len ++;
+			}
+		}
+		irc_reply( irc, 376, ":End of MOTD" );
+		close( fd );
+	}
+}
+
 void irc_topic( irc_t *irc, char *channel )
 {
 	if( strcasecmp( channel, irc->channel ) == 0 )
-		irc_reply( irc, 332, "%s :%s", irc->channel, CONTROL_TOPIC );
+	{
+		irc_reply( irc, 332, "%s :%s", channel, CONTROL_TOPIC );
+	}
 	else
-		irc_reply( irc, 331, "%s :No topic", irc->channel );
+	{
+		struct conversation *c = conv_findchannel( channel );
+		
+		if( c )
+			irc_reply( irc, 332, "%s :BitlBee groupchat: \"%s\". Please keep in mind that root-commands won't work here. Have fun!", channel, c->title );
+		else
+			irc_reply( irc, 331, "%s :No topic for this channel" );
+	}
 }
 
 void irc_whois( irc_t *irc, char *nick )
@@ -555,14 +763,14 @@ int irc_away( irc_t *irc, char *away )
 	if( away && *away )
 	{
 		irc_reply( irc, 306, ":You're now away: %s", away );
-		/* irc_umode_set( irc, irc->myhost, "+a" ); */
+		irc_umode_set( irc, irc->myhost, "+a" );
 		u->away = strdup( away );
 	}
 	else
 	{
 		if( u->away ) free( u->away );
 		u->away = NULL;
-		/* irc_umode_set( irc, irc->myhost, "-a" ); */
+		irc_umode_set( irc, irc->myhost, "-a" );
 		irc_reply( irc, 305, ":Welcome back" );
 	}
 	
@@ -579,32 +787,78 @@ int irc_away( irc_t *irc, char *away )
 
 void irc_spawn( irc_t *irc, user_t *u )
 {
-	if( user_find( irc, irc->nick ) )
-		irc_write( irc, ":%s!%s@%s JOIN :%s", u->nick, u->user, u->host, irc->channel );
-	if( strcasecmp( u->nick, irc->nick ) == 0 )
+	irc_join( irc, u, irc->channel );
+}
+
+void irc_join( irc_t *irc, user_t *u, char *channel )
+{
+	if( ( strcasecmp( channel, irc->channel ) != 0 ) || user_find( irc, irc->nick ) )
+		irc_write( irc, ":%s!%s@%s JOIN :%s", u->nick, u->user, u->host, channel );
+	
+	if( nick_cmp( u->nick, irc->nick ) == 0 )
 	{
-		irc_write( irc, ":%s MODE %s +%s", irc->myhost, irc->channel, CMODE );
-		irc_names( irc, irc->channel );
-		irc_topic( irc, irc->channel );
+		irc_write( irc, ":%s MODE %s +%s", irc->myhost, channel, CMODE );
+		irc_names( irc, channel );
+		irc_topic( irc, channel );
 	}
 }
 
+void irc_part( irc_t *irc, user_t *u, char *channel )
+{
+	irc_write( irc, ":%s!%s@%s PART %s", u->nick, u->user, u->host, channel );
+}
+
 void irc_kill( irc_t *irc, user_t *u )
 {
 	irc_write( irc, ":%s!%s@%s QUIT :%s", u->nick, u->user, u->host, "Leaving..." );
 }
 
-int irc_send( irc_t *irc, char *nick, char *s )
+void irc_invite( irc_t *irc, char *nick, char *channel )
 {
+	struct conversation *c = conv_findchannel( channel );
 	user_t *u = user_find( irc, nick );
 	
-	if( !u )
+	if( u && c && ( u->gc == c->gc ) )
+		if( c->gc && c->gc->prpl && c->gc->prpl->chat_invite )
+		{
+			c->gc->prpl->chat_invite( c->gc, c->id, "", u->handle );
+			irc_reply( irc, 341, "%s %s", nick, channel );
+			return;
+		}
+	
+	irc_reply( irc, 482, "%s :Invite impossible; User/Channel non-existent or incompatible", channel );
+}
+
+int irc_send( irc_t *irc, char *nick, char *s )
+{
+	struct conversation *c = NULL;
+	user_t *u = NULL;
+	
+	if( *nick == '#' )
 	{
-		if( irc->private )
-			irc_reply( irc, 401, "%s :Nick does not exist", nick );
-		else
-			irc_usermsg( irc, "Nick '%s' does not exist!", nick );
-		return( 0 );
+		if( !( c = conv_findchannel( nick ) ) )
+		{
+			irc_reply( irc, 403, "%s :Channel does not exist", nick );
+			return( 0 );
+		}
+	}
+	else
+	{
+		u = user_find( irc, nick );
+		
+		if( !u )
+		{
+			if( irc->private )
+				irc_reply( irc, 401, "%s :Nick does not exist", nick );
+			else
+				irc_usermsg( irc, "Nick '%s' does not exist!", nick );
+			return( 0 );
+		}
+		
+		u->private = irc->private;
+		
+		if( u->away )
+			irc_reply( irc, 301, "%s :%s", u->nick, u->away );
 	}
 	
 	if( *s == 1 && s[strlen(s)-1] == 1 )
@@ -627,43 +881,24 @@ int irc_send( irc_t *irc, char *nick, char *s )
 		}
 	}
 	
-	u->private = irc->private;
-	
-	if( u->away )
-		irc_reply( irc, 301, "%s :%s", u->nick, u->away );
-	
-	if( u->send_handler )
-		u->send_handler( irc, u, s );
+	if( u && u->send_handler )
+		return( u->send_handler( irc, u, s ) );
+	else if( c && c->gc && c->gc->prpl )
+		c->gc->prpl->chat_send( c->gc, c->id, s );
 	
 	return( 1 );
 }
 
 int buddy_send_handler( irc_t *irc, user_t *u, char *msg )
 {
-	if( !u ) return( 0 );
-	if( !u->gc ) return( 0 );
+	if( !u || !u->gc ) return( 0 );
 	return( ((struct gaim_connection *)u->gc)->prpl->send_im( u->gc, u->handle, msg, strlen( msg ), 0 ) );
 }
 
-int irc_msgfrom( irc_t *irc, char *nick, char *msg )
+int irc_privmsg( irc_t *irc, user_t *u, char *type, char *to, char *prefix, char *msg )
 {
-	user_t *u = user_find( irc, nick );
-	static char *prefix = NULL;
-	char *s = msg, *line = msg;
 	char last = 0;
-	
-	if( !u ) return( 0 );
-	if( prefix && *prefix ) free( prefix );
-	
-	if( !u->private && strcasecmp( u->nick, irc->mynick ) != 0 )
-	{
-		prefix = malloc( strlen( irc->nick) + 3 );
-		sprintf( prefix, "%s%s", irc->nick, set_getstr( irc, "to_char" ) );
-	}
-	else
-	{
-		prefix = "";
-	}
+	char *s = msg, *line = msg;
 	
 	/* The almighty linesplitter .. woohoo!! */
 	while( !last )
@@ -681,15 +916,15 @@ int irc_msgfrom( irc_t *irc, char *nick, char *msg )
 		}
 		if( *s == 0 )
 		{
-			if( strncasecmp( line, "/me ", 4 ) == 0 && u->private )
+			if( strncasecmp( line, "/me ", 4 ) == 0 && ( !prefix || !*prefix ) && strcasecmp( type, "PRIVMSG" ) == 0 )
 			{
-				irc_write( irc, ":%s!%s@%s PRIVMSG %s :\001ACTION %s\001", u->nick, u->user, u->host,
-				           irc->nick, line + 4 );
+				irc_write( irc, ":%s!%s@%s %s %s :\001ACTION %s\001", u->nick, u->user, u->host,
+				           type, to, line + 4 );
 			}
 			else
 			{
-				irc_write( irc, ":%s!%s@%s PRIVMSG %s :%s%s", u->nick, u->user, u->host,
-				           u->private ? irc->nick : irc->channel, prefix, line );
+				irc_write( irc, ":%s!%s@%s %s %s :%s%s", u->nick, u->user, u->host,
+				           type, to, prefix, line );
 			}
 			line = s + 1;
 		}
@@ -699,18 +934,37 @@ int irc_msgfrom( irc_t *irc, char *nick, char *msg )
 	return( 1 );
 }
 
-/* Sort-of same as irc_msgfrom, but limited because notices don't know
-   CTCP ACTION and because this one lacks multiline and in-channel support. */
+int irc_msgfrom( irc_t *irc, char *nick, char *msg )
+{
+	user_t *u = user_find( irc, nick );
+	static char *prefix = NULL;
+	
+	if( !u ) return( 0 );
+	if( prefix && *prefix ) free( prefix );
+	
+	if( !u->private && nick_cmp( u->nick, irc->mynick ) != 0 )
+	{
+		int len = strlen( irc->nick) + 3;
+		prefix = malloc( len );
+		snprintf( prefix, len, "%s%s", irc->nick, set_getstr( irc, "to_char" ) );
+		prefix[len-1] = 0;
+	}
+	else
+	{
+		prefix = "";
+	}
+	
+	return( irc_privmsg( irc, u, "PRIVMSG", u->private ? irc->nick : irc->channel, prefix, msg ) );
+}
+
 int irc_noticefrom( irc_t *irc, char *nick, char *msg )
 {
 	user_t *u = user_find( irc, nick );
-
+	
 	if( u )
-		irc_write( irc, ":%s!%s@%s NOTICE %s :%s", u->nick, u->user, u->host, irc->nick, msg );
+		return( irc_privmsg( irc, u, "NOTICE", irc->nick, "", msg ) );
 	else
 		return( 0 );
-	
-	return( 1 );
 }
 
 static char *set_eval_to_char( irc_t *irc, set_t *set, char *value )
@@ -729,22 +983,26 @@ static char *set_eval_ops( irc_t *irc, set_t *set, char *value )
 {
 	if( strcasecmp( value, "user" ) == 0 )
 	{
-		irc_write( irc, ":%s MODE %s %s %s %s", irc->myhost, irc->channel, "+o-o", irc->nick, irc->mynick );
+		irc_write( irc, ":%s!%s@%s MODE %s %s %s %s", irc->mynick, irc->mynick, irc->myhost,
+		                                              irc->channel, "+o-o", irc->nick, irc->mynick );
 		return( value );
 	}
 	else if( strcasecmp( value, "root" ) == 0 )
 	{
-		irc_write( irc, ":%s MODE %s %s %s %s", irc->myhost, irc->channel, "-o+o", irc->nick, irc->mynick );
+		irc_write( irc, ":%s!%s@%s MODE %s %s %s %s", irc->mynick, irc->mynick, irc->myhost,
+		                                              irc->channel, "-o+o", irc->nick, irc->mynick );
 		return( value );
 	}
 	else if( strcasecmp( value, "both" ) == 0 )
 	{
-		irc_write( irc, ":%s MODE %s %s %s %s", irc->myhost, irc->channel, "+oo", irc->nick, irc->mynick );
+		irc_write( irc, ":%s!%s@%s MODE %s %s %s %s", irc->mynick, irc->mynick, irc->myhost,
+		                                              irc->channel, "+oo", irc->nick, irc->mynick );
 		return( value );
 	}
 	else if( strcasecmp( value, "none" ) == 0 )
 	{
-		irc_write( irc, ":%s MODE %s %s %s %s", irc->myhost, irc->channel, "-oo", irc->nick, irc->mynick );
+		irc_write( irc, ":%s!%s@%s MODE %s %s %s %s", irc->mynick, irc->mynick, irc->myhost,
+		                                              irc->channel, "-oo", irc->nick, irc->mynick );
 		return( value );
 	}
 	
